#version 330

uniform samplerCube texUnit;
uniform float f = 0.05; 
uniform float power = 5.0;
uniform float bias = 0.3;
uniform float scale = 1.0;
uniform float eta = 0.66;

in vec3 normalV;
in vec3 eyeV;

out vec4 outColor;

void main() {

	// normalize vectors
	vec3 n = normalize(normalV);
	vec3 e = normalize(eyeV);

	// compute reflection vector
	vec3 t = reflect(e, n);

	// access cube map texture
	vec3 ref = texture(texUnit, t).rgb;

	// simple version
	float fresnelApprox = 1 - pow(dot(-e,n), power);
	
	// Schlick approximation
	float f =  ((1.0 - eta) * (1.0 - eta)) / 
				((1.0 + eta) * (1.0 + eta));
	float schlick = f + (1 - f) * pow(1 - dot(-e,n), power);
	
	// cgTut empirical approximation
	float R = max(0, min(1, bias + scale * pow(1.0 - dot(n,-e), power)));
	
	outColor = vec4(ref, R);
	// debug - show the reflection coefficient
	//outColor = vec4(vec3(fresnelApprox), 1);
}