#version 330

uniform samplerCube texUnit;
uniform float f = 0.05; 
uniform float power = 2.0;
uniform float bias = 0.3;
uniform float scale = 2.0;
uniform float eta = 0.66;

in vec3 normalV;
in vec3 eyeV;

out vec4 outColor;

void main() {

	// normalize vectors
	vec3 n = normalize(normalV);
	vec3 e = normalize(eyeV);

	// compute reflection vector
	vec3 trefl = reflect(e, n);
	
	// compute refraction vector
	vec3 trefr = refract(e, n, eta);

	// access cube map texture
	vec3 refl = texture(texUnit, trefl).rgb;
	vec3 refr = texture(texUnit, trefr).rgb;
	
	// simple version
	float fresnelApprox = 1 - pow(dot(-e,n), power);
	
	// schlick approximation
	float f =  ((1.0 - eta) * (1.0 - eta)) / 
				((1.0 + eta) * (1.0 + eta));
	float schlick = f + (1 - f) * pow(1 - dot(-e,n), 5.0);
	
	// CgTut empirical approximation
	float R = max(0, min(1, bias + scale * pow(1.0 - dot(n,-e), power)));
	
	outColor = vec4(mix(refr, refl, R), 1);
	// debug - show the reflection coefficient
	//outColor = vec4(vec3(fresnelApprox), 1);
}